if select(2, UnitClass("player")) ~= "DRUID" then
	return
end

SmartyCatManaBar = LibStub("AceAddon-3.0"):NewAddon("SmartyCatManaBar", "AceEvent-3.0", "AceConsole-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale("SmartyCatManaBar", true)

local SM, options

local defaults = {
	profile = {
		border = "Blizzard Tooltip",
		height = 10,
		width = 130,
		font = "Arial Narrow",
		fontsize = 11,
		texture = "Otravi",
		barcolor = { r = 0, g = 0, b = 1, a = 1 },
		bordercolor = { r = 0.5, g = 0.5, b = 0.5, a = 1 },
		backcolor = { r = 0, g = 0, b = 0, a = 0.5 },
		fontcolor = { r = 1, g = 1, b = 1 },
		hideInCaster = true,
		hideWhenFull = false,
		centerText = false,
		lStyle = 1,
		rStyle = 2,
		x = 0,
		y = 0,
        relativePoint = "CENTER",
		lock = false,
    }
}

local function GetLSMIndex(t, value)
	for k, v in pairs(SM:List(t)) do
	if v == value then
		return k
		end
	end
	return nil
end

function SmartyCatManaBar:OnInitialize()
	SM = LibStub('LibSharedMedia-3.0')
	self.db = LibStub("AceDB-3.0"):New("SmartyCatManaBarDB", defaults, "Default")
	
	SM:Register("statusbar", "Perl", "Interface\\AddOns\\SmartyCatManaBar\\perl")
	SM:Register("statusbar", "Charcoal", "Interface\\AddOns\\SmartyCatManaBar\\charcoal")
	SM:Register("statusbar", "Otravi", "Interface\\AddOns\\SmartyCatManaBar\\otravi")
	SM:Register("statusbar", "Smooth", "Interface\\AddOns\\SmartyCatManaBar\\smooth")
	SM:Register("statusbar", "Glaze", "Interface\\AddOns\\SmartyCatManaBar\\glaze")
	SM:Register("statusbar", "Granite", "Interface\\AddOns\\SmartyCatManaBar\\granite")
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SmartyCatManaBar", self:LoadOptions())
	self:RegisterChatCommand(L["scmb"], function() LibStub("AceConfigDialog-3.0"):Open("SmartyCatManaBar") end )
	local optFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("SmartyCatManaBar", "SmartyCatManaBar")
end

function SmartyCatManaBar:OnEnable()
	if not self.frame then
		self:SetupFrame()
	end
	
	self.db.RegisterCallback(SmartyCatManaBar, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(SmartyCatManaBar, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(SmartyCatManaBar, "OnProfileReset", "OnProfileChanged")
	self:RegisterEvent("UNIT_DISPLAYPOWER", "OnEvent")
	self:RegisterEvent("UNIT_MANA", "OnEvent")
	self:RegisterEvent("UNIT_MAXMANA", "OnEvent")
	SM.RegisterCallback(SmartyCatManaBar, "SharedMedia_SetGlobal", "UpdateLook")
	SM.RegisterCallback(SmartyCatManaBar, "LibSharedMedia_Registered", "UpdateLook")
	
	self:OnProfileChanged()
end

function SmartyCatManaBar:OnDisable()
	self.frame:Hide()
end

function SmartyCatManaBar:OnProfileChanged()
	self.frame:ClearAllPoints()
	self.frame:SetPoint(self.db.profile.relativePoint, UIParent, self.db.profile.relativePoint, self.db.profile.x, self.db.profile.y)
	self:UpdateLook()
	self:UpdateMana()
end

local textStyles = {
	[1] = function(currMana, maxMana)
		return currMana .. "/" .. maxMana
	end,
	[2] = function(currMana, maxMana)
		return ("%.0f%%"):format(currMana / maxMana * 100)
	end,
	[3] = function(currMana, maxMana)
		return ("%d/%d || %.0f%%"):format(currMana, maxMana, currMana / maxMana * 100)
	end,
	[4] = function()
		return "" 
	end,
}

local tStyles = {L["Absolute"], L["Percentage"], L["Absolute and Percentage"], L["Hide"]}

function SmartyCatManaBar:OnEvent(event, unitID)
	if unitID == "player" then
		self:UpdateMana()
	end
end

function SmartyCatManaBar:UpdateMana()
    local f = self.frame
    local db = self.db.profile
    local currMana = UnitPower("player", 0)
    local maxMana = UnitPowerMax("player", 0)
    
    if (db.hideInCaster and UnitPowerType("player") == 0) or (currMana == maxMana and db.hideWhenFull) then
		f:Hide()
	else
		f:Show()
		f.lTextField:SetText(textStyles[db.lStyle](currMana, maxMana))
		f.rTextField:SetText(textStyles[db.rStyle](currMana, maxMana))
		f.bar:SetValue(currMana / maxMana)
	end
end

function SmartyCatManaBar:LoadOptions()
	if not options then
		options = {
			name = "SmartyCatManaBar",
			type = "group",
			childGroups = "tab",
			args = {
				frame = {
					type = "group",
					name = L["Mana Bar"],
					desc = L["Set Mana Bar"],
					args = {
						height = {
							type = "range",
							name = L["Height"],
							desc = L["Set the bar height"],
							get = function() return SmartyCatManaBar.db.profile.height end,
							set = function(info, v)
								SmartyCatManaBar.db.profile.height = v
								SmartyCatManaBar:UpdateLook()
							end,
							min = 1, max = 50, step = 1,
							order = 1,
						},
						width = {
							type = "range",
							name = L["Width"],
							desc = L["Set the bar width"],
							get = function() return SmartyCatManaBar.db.profile.width end,
							set = function(info, v)
								SmartyCatManaBar.db.profile.width = v
								SmartyCatManaBar:UpdateLook()
							end,
							min = 80, max = 300, step = 2,
							order = 2,
						},
						border = {
							type = "select",
							name = L["Border"],
							desc = L["Set the border style"],
							get = function() return GetLSMIndex("border", SmartyCatManaBar.db.profile.border) end,
							set = function(info, v) 
								SmartyCatManaBar.db.profile.border = SM:List("border")[v]
								SmartyCatManaBar:UpdateLook() 
							end,
							values = SM:List("border"),
							order = 3,
						},
						bartexture = {
							type = "select", 
							name = L["Bar Texture"],
							desc = L["Set the bar texture"],
							get = function() return GetLSMIndex("statusbar", SmartyCatManaBar.db.profile.texture) end,
							set = function(info, v) 
								SmartyCatManaBar.db.profile.texture = SM:List("statusbar")[v]
								SmartyCatManaBar:UpdateLook()
							end,
							values = SM:List("statusbar"),
							order = 4,
						},
						font = {
							type = "select",
							name = L["Font"],
							desc = L["Set the font"],
							get = function() return GetLSMIndex("font", SmartyCatManaBar.db.profile.font) end,
							set = function(info, v) 
								SmartyCatManaBar.db.profile.font = SM:List("font")[v]
								SmartyCatManaBar:UpdateLook()
							end,
							values = SM:List("font"),
							order = 5,
						},
						fontsize = {
							type = "range",
							name = L["Font Size"],
							desc = L["Set the font size"],
							get = function() return SmartyCatManaBar.db.profile.fontsize end,
							set = function(info, v)
								SmartyCatManaBar.db.profile.fontsize = v
								SmartyCatManaBar:UpdateLook()
							end,
							min = 6, max = 26, step = 1,
							order = 6,
						},
						textStyles = {
							type = "group",
							name = L["Text Styles"],
							desc = L["Set the text styles"],
							order = 7,
							inline = true,
							args = {
								centerText = {
									type = "toggle", 
									name = L["Center Text"],
									desc = L["Center the text"],
									get = function() return SmartyCatManaBar.db.profile.centerText end,
									set = function(info, v) 
										SmartyCatManaBar.db.profile.centerText = v
										if v then
											SmartyCatManaBar.db.profile.lStyle = 4
											SmartyCatManaBar.db.profile.rStyle = 3
										else
											SmartyCatManaBar.db.profile.lStyle = 1
											SmartyCatManaBar.db.profile.rStyle = 2
										end 
										SmartyCatManaBar:UpdateLook()
										SmartyCatManaBar:UpdateMana()
									end,
									order = 1,
								},
								lText = {
									type = "select", 
									name = L["Left Text Style"],
									desc = L["Set the left text style"],
									get = function() return SmartyCatManaBar.db.profile.lStyle end,
									set = function(info, v) 
										SmartyCatManaBar.db.profile.lStyle = v 
										SmartyCatManaBar:UpdateMana()
									end,
									values = tStyles,
									hidden = function() return SmartyCatManaBar.db.profile.centerText end,
									order = 2,
								},
								rText = {
									type = "select", 
									name = L["Right Text Style"],
									desc = L["Set the right text style"],
									get = function() return SmartyCatManaBar.db.profile.rStyle end,
									set = function(info, v) 
										SmartyCatManaBar.db.profile.rStyle = v 
										SmartyCatManaBar:UpdateMana("player")
									end,
									values = tStyles,
									hidden = function() return SmartyCatManaBar.db.profile.centerText end,
									order = 3,
								},
								cText = {
									type = "select", 
									name = L["Center Text Style"],
									desc = L["Set the center text style"],
									get = function() return SmartyCatManaBar.db.profile.rStyle end,
									set = function(info, v) 
										SmartyCatManaBar.db.profile.rStyle = v 
										SmartyCatManaBar:UpdateMana()
									end,
									values = tStyles,
									hidden = function() return not SmartyCatManaBar.db.profile.centerText end,
									order = 4,
								},
							},
						},
						barcolor = {
							type = "color",
							name = L["Bar Color"],
							desc = L["Set the bar color"],
							get = function() return SmartyCatManaBar.db.profile.barcolor.r, SmartyCatManaBar.db.profile.barcolor.g, SmartyCatManaBar.db.profile.barcolor.b, SmartyCatManaBar.db.profile.barcolor.a end,
							set = function(info, r,g,b,a) 
								SmartyCatManaBar.db.profile.barcolor = {r=r, g=g, b=b, a=a}
								SmartyCatManaBar.frame.bar:SetStatusBarColor(r, g, b, a) 
							end,
							hasAlpha = true,
							order = 20,
						},
						bordercolor = {
							type = "color",
							name = L["Border Color"],
							desc = L["Set the border color"],
							get = function() return SmartyCatManaBar.db.profile.bordercolor.r, SmartyCatManaBar.db.profile.bordercolor.g, SmartyCatManaBar.db.profile.bordercolor.b, SmartyCatManaBar.db.profile.bordercolor.a end,
							set = function(info, r,g,b,a)
								SmartyCatManaBar.db.profile.bordercolor = {r=r, g=g, b=b, a=a}
								SmartyCatManaBar.frame:SetBackdropBorderColor(r, g, b, a)
							end,
							hasAlpha = true,
							order = 21,
						},
						backcolor = {
							type = "color",
							name = L["Background Color"], 
							desc = L["Set the background color"],
							get = function() return SmartyCatManaBar.db.profile.backcolor.r, SmartyCatManaBar.db.profile.backcolor.g, SmartyCatManaBar.db.profile.backcolor.b, SmartyCatManaBar.db.profile.backcolor.a end,
							set = function(info, r,g,b,a)
								SmartyCatManaBar.db.profile.backcolor = {r=r, g=g, b=b, a=a}
								SmartyCatManaBar.frame:SetBackdropColor(r, g, b, a)
							end,
							hasAlpha = true,
							order = 22,
						},
						fontcolor = {
							type = "color",
							name = L["Font Color"], 
							desc = L["Set the font color"],
							get = function() return SmartyCatManaBar.db.profile.fontcolor.r, SmartyCatManaBar.db.profile.fontcolor.g, SmartyCatManaBar.db.profile.fontcolor.b end,
							set = function(info, r,g,b)
								SmartyCatManaBar.db.profile.fontcolor = {r=r, g=g, b=b}
								SmartyCatManaBar.frame.lTextField:SetTextColor(r, g, b)
								SmartyCatManaBar.frame.rTextField:SetTextColor(r, g, b)
							end,
							order = 23,
						},
					},
				},
				hideInCaster = {
					type = "toggle",
					name = L["Hide in Caster Form"], 
					desc = L["Hides the mana bar when not in Cat or Bear form"],
					get = function() return SmartyCatManaBar.db.profile.hideInCaster end,
					set = function(info, v)
						SmartyCatManaBar.db.profile.hideInCaster = v
						SmartyCatManaBar:UpdateMana()
					end,
					order = 2,
				},
				hideWhenFull = {
					type = "toggle",
					name = L["Hide at Full Mana"], 
					desc = L["Hides the mana bar when mana is full"],
					get = function() return SmartyCatManaBar.db.profile.hideWhenFull end,
					set = function(info, v)
						SmartyCatManaBar.db.profile.hideWhenFull = v
						SmartyCatManaBar:UpdateMana()
					end,
					order = 3,
				},
				lock = {
					type = "toggle",
					name = L["Lock"],
					desc = L["Lock the mana bar"],
					get = function() return SmartyCatManaBar.db.profile.lock end,
					set = function(info, v)
						SmartyCatManaBar.db.profile.lock = v
					end,
					order = 1,
				},
			},
		}
		
		options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(SmartyCatManaBar.db)
	end
	
	return options
end

function SmartyCatManaBar:SetupFrame()
	local f = CreateFrame("Frame", "SCMBFrame", UIParent)
	
	f:SetClampedToScreen(true)
	
	f:SetMovable(true)
	f:EnableMouse(true)
	f:RegisterForDrag("LeftButton")
	f:SetScript("OnDragStart", function() if not self.db.profile.lock then this:StartMoving() end end)
	f:SetScript("OnDragStop", function() 
		this:StopMovingOrSizing() 
		local _, _, relativePoint, xOfs, yOfs = this:GetPoint()
		self.db.profile.relativePoint = relativePoint
		self.db.profile.x = floor(xOfs + 0.5)
		self.db.profile.y = floor(yOfs + 0.5)
	end)
	
	f.bar = CreateFrame("StatusBar", nil, f)
	f.bar:SetMinMaxValues(0, 1)
	f.lTextField = f.bar:CreateFontString(nil, "OVERLAY")
	f.rTextField = f.bar:CreateFontString(nil, "OVERLAY")
	f.bar:ClearAllPoints()
	f.bar:SetPoint("TOPLEFT", 4, -4)
	f.bar:SetPoint("BOTTOMRIGHT", -4, 4)
	
	self.frame = f
end

function SmartyCatManaBar:UpdateLook()
	local db = self.db.profile
	local font = SM:Fetch("font", db.font)
	local f = self.frame
	local inset = 4
	
	f:SetWidth(db.width+10)
	f:SetHeight(db.height+10)
	f:SetBackdrop({ 
		bgFile = SM:Fetch("statusbar", db.texture),
		edgeFile = SM:Fetch("border", db.border),
		edgeSize = 12,
		insets = {left = inset, right = inset, top = inset, bottom = inset},
	})
	f:SetBackdropColor(db.backcolor.r, db.backcolor.g, db.backcolor.b, db.backcolor.a)
	f:SetBackdropBorderColor(db.bordercolor.r, db.bordercolor.g, db.bordercolor.b, db.bordercolor.a) 
	
	f.bar:SetStatusBarTexture(SM:Fetch("statusbar", db.texture))
	f.bar:SetStatusBarColor(db.barcolor.r, db.barcolor.g, db.barcolor.b, db.barcolor.a)
	
	f.lTextField:SetFont(font, db.fontsize)
	f.lTextField:ClearAllPoints()
	f.lTextField:SetShadowColor(0.1, 0.1, 0.1)
	f.lTextField:SetShadowOffset(1,-1)
	f.lTextField:SetTextColor(db.fontcolor.r, db.fontcolor.g, db.fontcolor.b)
	f.lTextField:SetWidth(f:GetWidth()*0.7)
	f.lTextField:SetJustifyH("LEFT")
	f.lTextField:SetPoint("LEFT", f, "LEFT", 6, 0)
	
	f.rTextField:SetFont(font, db.fontsize)
	f.rTextField:ClearAllPoints()
	f.rTextField:SetShadowColor(0.1, 0.1, 0.1)
	f.rTextField:SetShadowOffset(1,-1)
	f.rTextField:SetTextColor(db.fontcolor.r, db.fontcolor.g, db.fontcolor.b)
	f.rTextField:SetWidth(f:GetWidth())
	if db.centerText then
		f.rTextField:SetJustifyH("CENTER")
		f.rTextField:SetPoint("CENTER", f, "CENTER")
	else
		f.rTextField:SetJustifyH("RIGHT")
		f.rTextField:SetPoint("RIGHT", f, "RIGHT", -6, 0)
	end
end